@@ -32,7 +32,11 @@ for item in os.listdir(parent):
 
 # Add any Sphinx extension module names here, as strings. They can be extensions
 # coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
-extensions = ['sphinx.ext.autodoc', 'sphinx.ext.doctest']
+extensions = [
+    'sphinx.ext.autodoc',
+    'sphinx.ext.doctest',
+    'repoze.sphinx.autointerface',
+]
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['.templates']
@@ -51,9 +55,10 @@ copyright = '2008, Agendaless Consulting'
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '1.0.16dev'
+
+version = '2.0a4'
 # The full version, including alpha/beta/rc tags.
-release = '1.0.16dev'
+release = version
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
