@@ -1,5 +1,6 @@
 import unittest
 
+
 class TestAuthTktCookiePlugin(unittest.TestCase):
     tempdir = None
     _now_testing = None
@@ -14,27 +15,36 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         if self._now_testing is not None:
             self._setNowTesting(self._now_testing)
 
+    def failUnless(self, predicate, message=''):
+        self.assertTrue(predicate, message) # Nannies go home!
+
+    def failIf(self, predicate, message=''):
+        self.assertFalse(predicate, message) # Nannies go home!
+
     def _getTargetClass(self):
         from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin
         return AuthTktCookiePlugin
 
     def _makeEnviron(self, kw=None):
-        environ = {'wsgi.version': (1,0)}
+        from wsgiref.util import setup_testing_defaults
+        environ = {}
+        setup_testing_defaults(environ)
         if kw is not None:
             environ.update(kw)
         environ['REMOTE_ADDR'] = '1.1.1.1'
-        environ['SERVER_NAME'] = 'localhost'
+        environ['HTTP_HOST'] = 'localhost'
         return environ
 
-    def _makeOne(self, *arg, **kw):
-        plugin = self._getTargetClass()(*arg, **kw)
+    def _makeOne(self, secret='s33kr3t', *arg, **kw):
+        plugin = self._getTargetClass()(secret, *arg, **kw)
         return plugin
 
     def _makeTicket(self, userid='userid', remote_addr='0.0.0.0',
                     tokens = [], userdata='userdata',
                     cookie_name='auth_tkt', secure=False,
                     time=None):
-        from paste.auth import auth_tkt
+        #from paste.auth import auth_tkt
+        import repoze.who._auth_tkt as auth_tkt
         ticket = auth_tkt.AuthTicket(
             'secret',
             userid,
@@ -50,12 +60,37 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         from repoze.who.plugins import auth_tkt
         auth_tkt._NOW_TESTING, self._now_testing = value, auth_tkt._NOW_TESTING
 
-    def test_implements(self):
+    def test_class_conforms_to_IIdentifier(self):
         from zope.interface.verify import verifyClass
         from repoze.who.interfaces import IIdentifier
         klass = self._getTargetClass()
         verifyClass(IIdentifier, klass)
 
+    def test_instance_conforms_to_IIdentifier(self):
+        from zope.interface.verify import verifyObject
+        from repoze.who.interfaces import IIdentifier
+        klass = self._getTargetClass()
+        verifyObject(IIdentifier, self._makeOne())
+
+    def test_class_conforms_to_IAuthenticator(self):
+        from zope.interface.verify import verifyClass
+        from repoze.who.interfaces import IAuthenticator
+        klass = self._getTargetClass()
+        verifyClass(IAuthenticator, klass)
+
+    def test_instance_conforms_to_IAuthenticator(self):
+        from zope.interface.verify import verifyObject
+        from repoze.who.interfaces import IAuthenticator
+        klass = self._getTargetClass()
+        verifyObject(IAuthenticator, self._makeOne())
+
+    def test_timeout_no_reissue(self):
+        self.assertRaises(ValueError, self._makeOne, 'userid', timeout=1)
+
+    def test_timeout_lower_than_reissue(self):
+        self.assertRaises(ValueError, self._makeOne, 'userid', timeout=1,
+                          reissue_time=2)
+
     def test_identify_nocookie(self):
         plugin = self._makeOne('secret')
         environ = self._makeEnviron()
@@ -69,7 +104,7 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         result = plugin.identify(environ)
         self.assertEqual(len(result), 4)
         self.assertEqual(result['tokens'], [''])
-        self.assertEqual(result['repoze.who.userid'], 'userid')
+        self.assertEqual(result['repoze.who.plugins.auth_tkt.userid'], 'userid')
         self.assertEqual(result['userdata'], 'userdata')
         self.failUnless('timestamp' in result)
         self.assertEqual(environ['REMOTE_USER_TOKENS'], [''])
@@ -83,7 +118,7 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         result = plugin.identify(environ)
         self.assertEqual(len(result), 4)
         self.assertEqual(result['tokens'], [''])
-        self.assertEqual(result['repoze.who.userid'], 'userid')
+        self.assertEqual(result['repoze.who.plugins.auth_tkt.userid'], 'userid')
         self.assertEqual(result['userdata'], 'userdata')
         self.failUnless('timestamp' in result)
         self.assertEqual(environ['REMOTE_USER_TOKENS'], [''])
@@ -97,7 +132,7 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         result = plugin.identify(environ)
         self.assertEqual(len(result), 4)
         self.assertEqual(result['tokens'], [''])
-        self.assertEqual(result['repoze.who.userid'], 1)
+        self.assertEqual(result['repoze.who.plugins.auth_tkt.userid'], 1)
         self.assertEqual(result['userdata'], 'userid_type:int')
         self.failUnless('timestamp' in result)
         self.assertEqual(environ['REMOTE_USER_TOKENS'], [''])
@@ -111,7 +146,7 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         result = plugin.identify(environ)
         self.assertEqual(len(result), 4)
         self.assertEqual(result['tokens'], [''])
-        self.assertEqual(result['repoze.who.userid'], 'userid')
+        self.assertEqual(result['repoze.who.plugins.auth_tkt.userid'], 'userid')
         self.assertEqual(result['userdata'], 'userid_type:unknown')
         self.failUnless('timestamp' in result)
         self.assertEqual(environ['REMOTE_USER_TOKENS'], [''])
@@ -132,6 +167,20 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         result = plugin.identify(environ)
         self.assertEqual(result, None)
 
+    def test_identify_with_checker_and_existing_account(self):
+        plugin = self._makeOne('secret', userid_checker=dummy_userid_checker)
+        val = self._makeTicket(userid='existing')
+        environ = self._makeEnviron({'HTTP_COOKIE':'auth_tkt=%s' % val})
+        result = plugin.identify(environ)
+        self.assertEqual(len(result), 4)
+        self.assertEqual(result['tokens'], [''])
+        self.assertEqual(result['repoze.who.plugins.auth_tkt.userid'], 'existing')
+        self.assertEqual(result['userdata'], 'userdata')
+        self.failUnless('timestamp' in result)
+        self.assertEqual(environ['REMOTE_USER_TOKENS'], [''])
+        self.assertEqual(environ['REMOTE_USER_DATA'],'userdata')
+        self.assertEqual(environ['AUTH_TYPE'],'cookie')
+
     def test_remember_creds_same(self):
         plugin = self._makeOne('secret')
         val = self._makeTicket(userid='userid')
@@ -140,6 +189,34 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
                                            'userdata':'userdata'})
         self.assertEqual(result, None)
 
+    def test_remember_creds_secure(self):
+        plugin = self._makeOne('secret', secure=True)
+        val = self._makeTicket(userid='userid', secure=True)
+        environ = self._makeEnviron()
+        result = plugin.remember(environ, {'repoze.who.userid':'userid',
+                                           'userdata':'userdata'})
+        self.assertEqual(len(result), 3)
+        self.assertEqual(result[0],
+                         ('Set-Cookie',
+                          'auth_tkt="%s"; '
+                          'Path=/; '
+                          'secure; '
+                          'HttpOnly' % val))
+        self.assertEqual(result[1],
+                         ('Set-Cookie',
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=localhost; '
+                           'secure; HttpOnly'
+                            % val))
+        self.assertEqual(result[2],
+                         ('Set-Cookie',
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=.localhost; '
+                           'secure; HttpOnly'
+                            % val))
+
     def test_remember_creds_different(self):
         plugin = self._makeOne('secret')
         old_val = self._makeTicket(userid='userid')
@@ -150,14 +227,46 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         self.assertEqual(len(result), 3)
         self.assertEqual(result[0],
                          ('Set-Cookie',
-                          'auth_tkt="%s"; Path=/' % new_val))
+                          'auth_tkt="%s"; '
+                          'Path=/' % new_val))
         self.assertEqual(result[1],
                          ('Set-Cookie',
-                           'auth_tkt="%s"; Path=/; Domain=localhost'
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=localhost'
                             % new_val))
         self.assertEqual(result[2],
                          ('Set-Cookie',
-                           'auth_tkt="%s"; Path=/; Domain=.localhost'
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=.localhost'
+                            % new_val))
+
+    def test_remember_creds_different_strips_port(self):
+        plugin = self._makeOne('secret')
+        old_val = self._makeTicket(userid='userid')
+        environ = self._makeEnviron({'HTTP_COOKIE':'auth_tkt=%s' % old_val,
+                                     'HTTP_HOST': 'localhost:8080',
+                                    })
+        new_val = self._makeTicket(userid='other', userdata='userdata')
+        result = plugin.remember(environ, {'repoze.who.userid':'other',
+                                           'userdata':'userdata'})
+        self.assertEqual(len(result), 3)
+        self.assertEqual(result[0],
+                         ('Set-Cookie',
+                          'auth_tkt="%s"; '
+                          'Path=/' % new_val))
+        self.assertEqual(result[1],
+                         ('Set-Cookie',
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=localhost'
+                            % new_val))
+        self.assertEqual(result[2],
+                         ('Set-Cookie',
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=.localhost'
                             % new_val))
 
     def test_remember_creds_different_include_ip(self):
@@ -172,14 +281,19 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         self.assertEqual(len(result), 3)
         self.assertEqual(result[0],
                          ('Set-Cookie',
-                          'auth_tkt="%s"; Path=/' % new_val))
+                          'auth_tkt="%s"; '
+                          'Path=/' % new_val))
         self.assertEqual(result[1],
                          ('Set-Cookie',
-                           'auth_tkt="%s"; Path=/; Domain=localhost'
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=localhost'
                             % new_val))
         self.assertEqual(result[2],
                          ('Set-Cookie',
-                           'auth_tkt="%s"; Path=/; Domain=.localhost'
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=.localhost'
                             % new_val))
 
     def test_remember_creds_different_bad_old_cookie(self):
@@ -192,39 +306,78 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         self.assertEqual(len(result), 3)
         self.assertEqual(result[0],
                          ('Set-Cookie',
-                          'auth_tkt="%s"; Path=/' % new_val))
+                          'auth_tkt="%s"; '
+                          'Path=/' % new_val))
         self.assertEqual(result[1],
                          ('Set-Cookie',
-                           'auth_tkt="%s"; Path=/; Domain=localhost'
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=localhost'
                             % new_val))
         self.assertEqual(result[2],
                          ('Set-Cookie',
-                           'auth_tkt="%s"; Path=/; Domain=.localhost'
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=.localhost'
                             % new_val))
 
-    def test_remember_creds_different_with_nonstring_tokens(self):
+    def test_remember_creds_different_with_tokens(self):
         plugin = self._makeOne('secret')
         old_val = self._makeTicket(userid='userid')
         environ = self._makeEnviron({'HTTP_COOKIE':'auth_tkt=%s' % old_val})
-        new_val = self._makeTicket(userid='other',
+        new_val = self._makeTicket(userid='userid',
                                    userdata='userdata',
-                                   tokens='foo,bar',
+                                   tokens=['foo', 'bar'],
                                   )
-        result = plugin.remember(environ, {'repoze.who.userid': 'other',
+        result = plugin.remember(environ, {'repoze.who.userid': 'userid',
                                            'userdata': 'userdata',
                                            'tokens': ['foo', 'bar'],
                                           })
         self.assertEqual(len(result), 3)
         self.assertEqual(result[0],
                          ('Set-Cookie',
-                          'auth_tkt="%s"; Path=/' % new_val))
+                          'auth_tkt="%s"; '
+                          'Path=/' % new_val))
         self.assertEqual(result[1],
                          ('Set-Cookie',
-                           'auth_tkt="%s"; Path=/; Domain=localhost'
+                           'auth_tkt="%s"; '
+                           'Path=/; Domain=localhost'
                             % new_val))
         self.assertEqual(result[2],
                          ('Set-Cookie',
-                           'auth_tkt="%s"; Path=/; Domain=.localhost'
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=.localhost'
+                            % new_val))
+
+    def test_remember_creds_different_with_tuple_tokens(self):
+        plugin = self._makeOne('secret')
+        old_val = self._makeTicket(userid='userid')
+        environ = self._makeEnviron({'HTTP_COOKIE':'auth_tkt=%s' % old_val})
+        new_val = self._makeTicket(userid='userid',
+                                   userdata='userdata',
+                                   tokens=['foo', 'bar'],
+                                  )
+        result = plugin.remember(environ, {'repoze.who.userid': 'userid',
+                                           'userdata': 'userdata',
+                                           'tokens': ('foo', 'bar'),
+                                          })
+        self.assertEqual(len(result), 3)
+        self.assertEqual(result[0],
+                         ('Set-Cookie',
+                          'auth_tkt="%s"; '
+                          'Path=/' % new_val))
+        self.assertEqual(result[1],
+                         ('Set-Cookie',
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=localhost'
+                            % new_val))
+        self.assertEqual(result[2],
+                         ('Set-Cookie',
+                           'auth_tkt="%s"; '
+                           'Path=/; '
+                           'Domain=.localhost'
                             % new_val))
 
     def test_remember_creds_different_int_userid(self):
@@ -241,6 +394,10 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
                           'auth_tkt="%s"; Path=/' % new_val))
 
     def test_remember_creds_different_long_userid(self):
+        try:
+            long
+        except NameError: #pragma NO COVER Python >= 3.0
+            return
         plugin = self._makeOne('secret')
         old_val = self._makeTicket(userid='userid')
         environ = self._makeEnviron({'HTTP_COOKIE':'auth_tkt=%s' % old_val})
@@ -256,9 +413,13 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
         plugin = self._makeOne('secret')
         old_val = self._makeTicket(userid='userid')
         environ = self._makeEnviron({'HTTP_COOKIE':'auth_tkt=%s' % old_val})
-        userid = unicode('\xc2\xa9', 'utf-8')
+        userid = b'\xc2\xa9'.decode('utf-8')
+        if type(b'') == type(''):
+            userdata = 'userid_type:unicode'
+        else: # XXX
+            userdata = ''
         new_val = self._makeTicket(userid=userid.encode('utf-8'),
-                                   userdata='userid_type:unicode')
+                                   userdata=userdata)
         result = plugin.remember(environ, {'repoze.who.userid':userid,
                                            'userdata':''})
         self.assertEqual(type(result[0][1]), str)
@@ -282,6 +443,37 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
                          ('Set-Cookie',
                           'auth_tkt="%s"; Path=/' % new_val))
 
+    def test_remember_max_age(self):
+        plugin = self._makeOne('secret')
+        environ = {'HTTP_HOST':'example.com'}
+        
+        tkt = self._makeTicket(userid='chris', userdata='')
+        result = plugin.remember(environ, {'repoze.who.userid':'chris',
+                                           'max_age':'500'})
+        
+        name,value = result.pop(0)
+        self.assertEqual('Set-Cookie', name)
+        self.failUnless(
+            value.startswith('auth_tkt="%s"; Path=/; Max-Age=500' % tkt),
+            value)
+        self.failUnless('; Expires=' in value)
+        
+        name,value = result.pop(0)
+        self.assertEqual('Set-Cookie', name)
+        self.failUnless(
+            value.startswith(
+            'auth_tkt="%s"; Path=/; Domain=example.com; Max-Age=500'
+            % tkt), value)
+        self.failUnless('; Expires=' in value)
+
+        name,value = result.pop(0)
+        self.assertEqual('Set-Cookie', name)
+        self.failUnless(
+            value.startswith(
+            'auth_tkt="%s"; Path=/; Domain=.example.com; Max-Age=500' % tkt),
+            value)
+        self.failUnless('; Expires=' in value)
+
     def test_forget(self):
         from datetime import datetime
         now = datetime(2009, 11, 5, 16, 15, 22)
@@ -312,6 +504,25 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
                          'Max-Age=0; Expires=Thu, 05 Nov 2009 16:15:22'
                         )
 
+    def test_authenticate_non_auth_tkt_credentials(self):
+        plugin = self._makeOne()
+        self.assertEqual(plugin.authenticate(environ={}, identity={}), None)
+    
+    def test_authenticate_without_checker(self):
+        plugin = self._makeOne()
+        identity = {'repoze.who.plugins.auth_tkt.userid': 'phred'}
+        self.assertEqual(plugin.authenticate({}, identity), 'phred')
+    
+    def test_authenticate_with_checker_and_non_existing_account(self):
+        plugin = self._makeOne('secret', userid_checker=dummy_userid_checker)
+        identity = {'repoze.who.plugins.auth_tkt.userid': 'phred'}
+        self.assertEqual(plugin.authenticate({}, identity), None)
+    
+    def test_authenticate_with_checker_and_existing_account(self):
+        plugin = self._makeOne('secret', userid_checker=dummy_userid_checker)
+        identity = {'repoze.who.plugins.auth_tkt.userid': 'existing'}
+        self.assertEqual(plugin.authenticate({}, identity), 'existing')
+
     def test_factory_wo_secret_wo_secretfile_raises_ValueError(self):
         from repoze.who.plugins.auth_tkt import make_plugin
         self.assertRaises(ValueError, make_plugin)
@@ -358,52 +569,19 @@ class TestAuthTktCookiePlugin(unittest.TestCase):
             userid_checker='repoze.who.plugins.auth_tkt:make_plugin')
         self.assertEqual(plugin.userid_checker, make_plugin)
 
-    def test_timeout_no_reissue(self):
-        self.assertRaises(ValueError, self._makeOne, 'userid', timeout=1)
-
-    def test_timeout_lower_than_reissue(self):
-        self.assertRaises(ValueError, self._makeOne, 'userid', timeout=1,
-                          reissue_time=2)
-
-    def test_identify_with_checker_and_existing_account(self):
-        plugin = self._makeOne('secret', userid_checker=dummy_userid_checker)
-        val = self._makeTicket(userid='existing')
-        environ = self._makeEnviron({'HTTP_COOKIE':'auth_tkt=%s' % val})
-        result = plugin.identify(environ)
-        self.assertEqual(len(result), 4)
-        self.assertEqual(result['tokens'], [''])
-        self.assertEqual(result['repoze.who.userid'], 'existing')
-        self.assertEqual(result['userdata'], 'userdata')
-        self.failUnless('timestamp' in result)
-        self.assertEqual(environ['REMOTE_USER_TOKENS'], [''])
-        self.assertEqual(environ['REMOTE_USER_DATA'],'userdata')
-        self.assertEqual(environ['AUTH_TYPE'],'cookie')
-    
-    def test_identify_with_checker_and_non_existing_account(self):
-        plugin = self._makeOne('secret', userid_checker=dummy_userid_checker)
-        val = self._makeTicket(userid='nonexisting')
-        environ = self._makeEnviron({'HTTP_COOKIE':'auth_tkt=%s' % val})
-        original_environ = environ.copy()
-        result = plugin.identify(environ)
-        self.assertEqual(result, None)
-        # The environ must not have been modified, excuding the paste.cookies
-        # variable:
-        del environ['paste.cookies']
-        self.assertEqual(environ, original_environ)
-
-    def test_remember_max_age(self):
+    def test_remember_max_age_unicode(self):
+        from repoze.who._compat import u
         plugin = self._makeOne('secret')
         environ = {'HTTP_HOST':'example.com'}
-        
         tkt = self._makeTicket(userid='chris', userdata='')
-        result = plugin.remember(environ, {'repoze.who.userid':'chris',
-                                           'max_age':'500'})
-        
-        name,value = result.pop(0)
+        result = plugin.remember(environ, {'repoze.who.userid': 'chris',
+                                           'max_age': u('500')})
+        name, value = result.pop(0)
         self.assertEqual('Set-Cookie', name)
+        self.failUnless(isinstance(value, str))
         self.failUnless(
             value.startswith('auth_tkt="%s"; Path=/; Max-Age=500' % tkt),
-            value)
+            (value, tkt))
         self.failUnless('; Expires=' in value)
         
         name,value = result.pop(0)
